pub struct CCtx(/* private fields */);Implementations
Sourceimpl CCtx
impl CCtx
pub fn new() -> Self
pub fn set_parameter(&mut self, p: ZL_CParam, v: i32) -> Result<(), Error>
Sourcepub fn ref_compressor(&mut self, compressor: &Compressor) -> Result<(), Error>
pub fn ref_compressor(&mut self, compressor: &Compressor) -> Result<(), Error>
Reference a Compressor for graph-based typed compression.
This enables TypedRef compression by associating the CCtx with a Compressor that has registered compression graphs.
IMPORTANT: The Compressor must remain valid for the duration of its usage. The Compressor must be validated before being referenced.